// Copyright 2024 Matthew Rich <matthewrich.conf@gmail.com>. All rights reserved.
 
package feudal

import(
  "feudal/message"
  "feudal/identity"
)

type Transporter interface {
  Transport(m message.Envelope)
}

type WorkerRouter interface {
  message.Sender
  Id() identity.Identifier
  Type() string
  State() string
  Stop()
}

type Factory func(name string, ruler WorkerRouter) Worker

type Creator interface {
  WorkerWhence(Factory, string) WorkerRouter
}

type Division interface {
  Join(w Worker)
  Disolve()
}

type Worker interface {
  message.Receiver
  SetContext(c interface{})
}

type Subjects interface {
  Add(name string, worker WorkerRouter)
  Remove(name string)
  Get(name string) WorkerRouter
  message.Sender
  Disolve()  
}

type Context interface {
  Creator
  message.Receiver
  Disolve()
  Self() WorkerRouter
  Subjects() Subjects
  Trigger(transition string)
  Subscribers(topic string) Subjects
}

type Audience interface {
  SwearFealty(Sovereign)
  Sovereign() Sovereign
}

type Sovereign interface {
  Creator
  Subjects() Subjects
}

/*

// provides interfaces for managing the worker tree (organization)
type Context interface {
  Creator
  System() System
  Ruler() feudal.Worker
  Subjects() Subjects
}

type System interface {
  identity.Addresser
  Creator
  Terminator
  WorkersPath(path string) subjects.Selector
}

*/
